class EkaCareError(Exception):
    """Base exception for all Eka Care SDK errors."""
    pass


class EkaCareAPIError(EkaCareError):
    """Exception raised when the Eka Care API returns an error."""
    pass


class EkaCareAuthError(EkaCareError):
    """Exception raised when authentication with the Eka Care API fails."""
    pass


class EkaCareValidationError(EkaCareError):
    """Exception raised when input validation fails."""
    pass


class EkaCareResourceNotFoundError(EkaCareError):
    """Exception raised when a requested resource is not found."""
    pass



from typing import Dict, Any


class Auth:
    """Authentication handler for Eka Care API."""
    
    def __init__(self, client):
        self.client = client
        
    def login(self) -> Dict[str, Any]:
        """
        Get an access token using client credentials.
        
        Returns:
            dict: Token response containing access_token, refresh_token, etc.
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> token_response = client.auth.login()
            >>> print(token_response["access_token"])
        """
        login_json = {
                "client_id": self.client.client_id,
                "client_secret": self.client.client_secret
            }
        if self.client.api_key is not None:
            login_json["api_key"] = self.client.api_key

        response = self.client.request(
            method="POST",
            endpoint="/connect-auth/v1/account/login",
            json=login_json,
            auth_required=False
        )
        return response
        
    def refresh_token(self, refresh_token: str) -> Dict[str, Any]:
        """
        Refresh an access token using a refresh token.
        
        Args:
            refresh_token (str): The refresh token
            
        Returns:
            dict: Token response containing new access_token, refresh_token, etc.
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> new_tokens = client.auth.refresh_token("your_refresh_token")
            >>> client.set_access_token(new_tokens["access_token"])
        """
        response = self.client.request(
            method="POST",
            endpoint="/connect-auth/v1/account/refresh",
            json={
                "refresh_token": refresh_token
            },
            auth_required=False
        )
        return response



from typing import Dict, Any


class AppointmentWebhook:
    """Client for interacting with Eka Appointment Webhooks"""

    def __init__(self, client):
        self.client = client

    def get_detailed_appointment_data(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        Fetch the details of a specific appointment.

        Args:
            payload (dict): The payload received from the webhook

        Returns:
            dict: Detailed appointment data
        """
        appointment_id = payload.get("data", {}).get("appointment_id")
        patient_id = payload.get("data", {}).get("patient_id")
        clinic_id = payload.get("data", {}).get("clinic_id")
        doctor_id = payload.get("data", {}).get("doctor_id")

        if not appointment_id:
            raise ValueError("Missing appointment_id in payload")
        if not patient_id:
            raise ValueError("Missing patient_id in payload")
        if not clinic_id:
            raise ValueError("Missing clinic_id in payload")
        if not doctor_id:
            raise ValueError("Missing doctor_id in payload")

        appointment_details = self.client.appointments.get_appointment_details(
            appointment_id)
        appointment_details["rescheduled"] = False
        old_aid = payload.get("data", {}).get("p_aid")
        if old_aid and isinstance(old_aid, str):
            appointment_details["rescheduled"] = True
            appointment_details["old_appointment_details"] = self.client.appointments.get_appointment_details(
                old_aid)

        return {
            "appointment_details": appointment_details,
            "patient_details": self.client.patient.get_patient(patient_id),
            "clinic_details": self.client.clinic_doctor.get_clinic_details(clinic_id),
            "doctor_details": self.client.clinic_doctor.get_doctor_details(doctor_id)
        }



# API Endpoints
API_URL_PRODUCTION = "https://api.eka.care"
API_URL_SANDBOX = "https://api.dev.eka.care"

# Document Types
DOCUMENT_TYPE_LAB_REPORT = "lr"
DOCUMENT_TYPE_PRESCRIPTION = "ps"
DOCUMENT_TYPE_DISCHARGE_SUMMARY = "ds"
DOCUMENT_TYPE_VACCINE_CERTIFICATE = "vc"
DOCUMENT_TYPE_INSURANCE = "in"
DOCUMENT_TYPE_INVOICE = "iv"
DOCUMENT_TYPE_SCAN = "sc"
DOCUMENT_TYPE_OTHER = "ot"

# Webhook Event Types
WEBHOOK_EVENT_APPOINTMENT_CREATED = "appointment.created"
WEBHOOK_EVENT_APPOINTMENT_UPDATED = "appointment.updated"
WEBHOOK_EVENT_PRESCRIPTION_CREATED = "prescription.created"
WEBHOOK_EVENT_PRESCRIPTION_UPDATED = "prescription.updated"

# ABDM Consent Status
CONSENT_STATUS_REQUESTED = "requested"
CONSENT_STATUS_GRANTED = "granted"
CONSENT_STATUS_EXPIRED = "expired"
CONSENT_STATUS_DENIED = "denied"
CONSENT_STATUS_REVOKED = "revoked"

# ABDM Health Information Types
HI_TYPE_OP_CONSULTATION = "OPConsultation"
HI_TYPE_PRESCRIPTION = "Prescription"
HI_TYPE_DISCHARGE_SUMMARY = "DischargeSummary"
HI_TYPE_DIAGNOSTIC_REPORT = "DiagnosticReport"
HI_TYPE_IMMUNIZATION_RECORD = "ImmunizationRecord"
HI_TYPE_HEALTH_DOCUMENT_RECORD = "HealthDocumentRecord"
HI_TYPE_WELLNESS_RECORD = "WellnessRecord"

# Vital LOINC Codes
LOINC_HEART_RATE = "8867-4"
LOINC_BLOOD_GLUCOSE_RANDOM = "74774-1"
LOINC_BLOOD_GLUCOSE_FASTING = "1558-6"
LOINC_BLOOD_GLUCOSE_AFTER_FOOD = "1521-4"
LOINC_BLOOD_OXYGEN = "2708-6"
LOINC_SYSTOLIC_BP = "8480-6"
LOINC_DIASTOLIC_BP = "8462-4"
LOINC_HEIGHT = "8302-2"
LOINC_BMI = "39156-5"



from typing import Dict, Any, Optional


class Profile:
    """Client for interacting with ABDM Profile APIs."""
    
    def __init__(self, client):
        self.client = client
    
    def get_profile(self) -> Dict[str, Any]:
        """
        Fetch the ABHA profile of the user.
        
        Returns:
            dict: ABHA profile details
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> profile = client.abdm_profile.get_profile()
            >>> print(profile["abha_address"])
        """
        return self.client.request(
            method="GET",
            endpoint="/abdm/v1/profile"
        )
    
    def update_profile(self, profile_data: Dict[str, Any]) -> Dict[str, Any]:
        """
        Update the ABHA profile.
        
        Args:
            profile_data (dict): Profile data to update
            
        Returns:
            dict: Response indicating success
            
        Note:
            Fields like first_name, gender, date_of_birth cannot be modified for
            Aadhaar-based KYC verified profiles.
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> profile_data = {
            ...     "address": "123 Main St",
            ...     "pincode": "110001"
            ... }
            >>> client.abdm_profile.update_profile(profile_data)
        """
        return self.client.request(
            method="PATCH",
            endpoint="/abdm/v1/profile",
            json=profile_data
        )
    
    def delete_profile(self) -> Dict[str, Any]:
        """
        Delete the ABHA profile.
        
        Returns:
            dict: Response indicating success
            
        Warning:
            This permanently removes the ABHA profile with all associated documents and records.
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> client.abdm_profile.delete_profile()
        """
        return self.client.request(
            method="DELETE",
            endpoint="/abdm/v1/profile"
        )
    
    def get_abha_card(self) -> bytes:
        """
        Get the ABHA card as an image.
        
        Returns:
            bytes: PNG image data of the ABHA card
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> card_image = client.abdm_profile.get_abha_card()
            >>> with open("abha_card.png", "wb") as f:
            ...     f.write(card_image)
        """
        return self.client.request(
            method="GET",
            endpoint="/abdm/v1/profile/asset/card",
            raw_response=True
        ).content
    
    def get_abha_qr_code(self, format: str = "json") -> Dict[str, Any]:
        """
        Get the data for ABHA QR code display.
        
        Args:
            format (str): Response format (currently only "json" is supported)
            
        Returns:
            dict: QR code data
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> qr_data = client.abdm_profile.get_abha_qr_code()
            >>> # Use qr_data to generate a QR code image
        """
        return self.client.request(
            method="GET",
            endpoint="/abdm/v1/profile/asset/qr",
            params={"format": format}
        )
    
    def initiate_kyc(self, method: str, identifier: str) -> Dict[str, Any]:
        """
        Initiate the KYC process.
        
        Args:
            method (str): KYC method ("abha-number" or "aadhaar")
            identifier (str): ABHA number or Aadhaar number
            
        Returns:
            dict: Response containing transaction ID
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> response = client.abdm_profile.initiate_kyc(
            ...     method="abha-number",
            ...     identifier="1234-5678-9012"
            ... )
            >>> txn_id = response["txn_id"]
        """
        return self.client.request(
            method="POST",
            endpoint="/abdm/v1/profile/kyc/init",
            json={
                "method": method,
                "identifier": identifier
            }
        )
    
    def verify_kyc_otp(self, txn_id: str, otp: str) -> Dict[str, Any]:
        """
        Verify OTP for KYC.
        
        Args:
            txn_id (str): Transaction ID from initiate_kyc
            otp (str): OTP received
            
        Returns:
            dict: Response indicating success
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> client.abdm_profile.verify_kyc_otp(
            ...     txn_id="transaction123",
            ...     otp="123456"
            ... )
        """
        return self.client.request(
            method="POST",
            endpoint="/abdm/v1/profile/kyc/verify",
            json={
                "txn_id": txn_id,
                "otp": otp
            }
        )
    
    def resend_kyc_otp(self, txn_id: str) -> Dict[str, Any]:
        """
        Resend OTP for KYC.
        
        Args:
            txn_id (str): Transaction ID from initiate_kyc
            
        Returns:
            dict: Response containing the new transaction ID
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> response = client.abdm_profile.resend_kyc_otp("transaction123")
            >>> new_txn_id = response["txn_id"]
        """
        return self.client.request(
            method="POST",
            endpoint="/abdm/v1/profile/kyc/resend",
            json={"txn_id": txn_id}
        )



from typing import Dict, Any, List


class Vitals:
    """Client for interacting with Eka Care Vitals API."""
    
    def __init__(self, client):
        self.client = client
        
    def update_vitals(self, txn_id: str, vitals_data: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Update patient vitals.
        
        Args:
            txn_id (str): Transaction ID for the vitals update
            vitals_data (list): List of vital records to update
            
        Returns:
            dict: Response indicating success or failure
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> heart_rate = client.vitals.create_heart_rate_vital(75, "2023-01-01T12:00:00")
            >>> blood_glucose = client.vitals.create_blood_glucose_vital(120, "2023-01-01T08:00:00", "fasting")
            >>> response = client.vitals.update_vitals("txn123", [heart_rate, blood_glucose])
        """
        return self.client.request(
            method="PUT",
            endpoint=f"/api/v1/vitals/{txn_id}",
            json={"vitals": vitals_data}
        )
        
    def create_heart_rate_vital(
        self,
        value: float,
        measured_at: str,
        unit: str = "{Counts}/min"
    ) -> Dict[str, Any]:
        """
        Create a heart rate vital record.
        
        Args:
            value (float): Heart rate value
            measured_at (str): Timestamp of measurement (ISO format)
            unit (str, optional): Unit of measurement
            
        Returns:
            dict: Vital record ready for submission
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> heart_rate = client.vitals.create_heart_rate_vital(75, "2023-01-01T12:00:00")
        """
        return {
            "vital_display_name": "heart_rate",
            "vital_loinc_code": "8867-4",
            "vital_value": {
                "vital_numeric_value": value,
                "vital_boolean_value": None,
                "vital_string_value": None
            },
            "unit_display_name": unit,
            "unit_ucum_code": "<ucum code>",
            "measured_at": {
                "DateTime": measured_at,
                "instant": None,
                "period": {
                    "startDateTime": None,
                    "endDateTime": None
                }
            }
        }
        
    def create_blood_glucose_vital(
        self,
        value: float,
        measured_at: str,
        glucose_type: str = "random",
        unit: str = "mg/dL"
    ) -> Dict[str, Any]:
        """
        Create a blood glucose vital record.
        
        Args:
            value (float): Blood glucose value
            measured_at (str): Timestamp of measurement (ISO format)
            glucose_type (str, optional): Type of glucose measurement
                (random, fasting, after_food)
            unit (str, optional): Unit of measurement
            
        Returns:
            dict: Vital record ready for submission
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> glucose = client.vitals.create_blood_glucose_vital(120, "2023-01-01T08:00:00", "fasting")
        """
        vital_types = {
            "random": {
                "name": "random_blood_glucose",
                "loinc_code": "74774-1"
            },
            "fasting": {
                "name": "fasting_blood_glucose",
                "loinc_code": "1558-6"
            },
            "after_food": {
                "name": "after_food_blood_glucose",
                "loinc_code": "1521-4"
            }
        }
        
        vital_type = vital_types.get(glucose_type, vital_types["random"])
        
        return {
            "vital_display_name": vital_type["name"],
            "vital_loinc_code": vital_type["loinc_code"],
            "vital_value": {
                "vital_numeric_value": value,
                "vital_boolean_value": None,
                "vital_string_value": None
            },
            "unit_display_name": unit,
            "unit_ucum_code": unit,
            "measured_at": {
                "DateTime": measured_at,
                "instant": None,
                "period": {
                    "startDateTime": None,
                    "endDateTime": None
                }
            }
        }
        
    def create_blood_oxygen_vital(
        self,
        value: float,
        measured_at: str,
        unit: str = "%"
    ) -> Dict[str, Any]:
        """
        Create a blood oxygen (SpO2) vital record.
        
        Args:
            value (float): Blood oxygen saturation value
            measured_at (str): Timestamp of measurement (ISO format)
            unit (str, optional): Unit of measurement
            
        Returns:
            dict: Vital record ready for submission
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> oxygen = client.vitals.create_blood_oxygen_vital(98, "2023-01-01T12:30:00")
        """
        return {
            "vital_display_name": "blood_oxygen",
            "vital_loinc_code": "2708-6",
            "vital_value": {
                "vital_numeric_value": value,
                "vital_boolean_value": None,
                "vital_string_value": None
            },
            "unit_display_name": unit,
            "unit_ucum_code": unit,
            "measured_at": {
                "DateTime": measured_at,
                "instant": None,
                "period": {
                    "startDateTime": None,
                    "endDateTime": None
                }
            }
        }
        
    def create_blood_pressure_vital(
        self,
        systolic: float,
        diastolic: float,
        measured_at: str,
        unit: str = "mm Hg"
    ) -> List[Dict[str, Any]]:
        """
        Create blood pressure vital records (systolic and diastolic).
        
        Args:
            systolic (float): Systolic blood pressure value
            diastolic (float): Diastolic blood pressure value
            measured_at (str): Timestamp of measurement (ISO format)
            unit (str, optional): Unit of measurement
            
        Returns:
            list: List of vital records ready for submission
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> bp_vitals = client.vitals.create_blood_pressure_vital(120, 80, "2023-01-01T14:45:00")
        """
        systolic_vital = {
            "vital_display_name": "systolic_bp",
            "vital_loinc_code": "8480-6",
            "vital_value": {
                "vital_numeric_value": systolic,
                "vital_boolean_value": None,
                "vital_string_value": None
            },
            "unit_display_name": unit,
            "unit_ucum_code": unit,
            "measured_at": {
                "DateTime": measured_at,
                "instant": None,
                "period": {
                    "startDateTime": None,
                    "endDateTime": None
                }
            }
        }
        
        diastolic_vital = {
            "vital_display_name": "diastolic_bp",
            "vital_loinc_code": "8462-4",
            "vital_value": {
                "vital_numeric_value": diastolic,
                "vital_boolean_value": None,
                "vital_string_value": None
            },
            "unit_display_name": unit,
            "unit_ucum_code": unit,
            "measured_at": {
                "DateTime": measured_at,
                "instant": None,
                "period": {
                    "startDateTime": None,
                    "endDateTime": None
                }
            }
        }
        
        return [systolic_vital, diastolic_vital]



from typing import Dict, Any


class Patient:
    """Client for interacting with Patient in Eka Ecosystem"""
    def __init__(self, client):
        self.client = client

    def get_patient(self, patient_id: str) -> Dict[str, Any]:
        """
        Fetch the details of a specific patient.

        Args:
            patient_id (str): The ID of the patient to fetch

        Returns:
            dict: Patient details

        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> patient = client.patient.get_patient("YOUR_PATIENT_ID")
            >>> print(patient)
        """
        return self.client.request(
            method="GET",
            endpoint=f"/dr/v1/patient/{patient_id}"
        )


from typing import Dict, Any
from io import BytesIO
import requests
import json
import os
from urllib.parse import urlencode, urlparse

class EkaScribeError(Exception):
    """Upload related errors for EkaScribe"""
    pass

class V2RX:
    """
    Client for interacting with V2RX (Ekascribe) session status APIs.
    This is typically used to fetch the results of an audio transcription job
    initiated via the file upload mechanism with action 'ekascribe'.
    """

    def __init__(self, client):
        """
        Initialize the V2RX client.

        Args:
            client: The EkaCareClient instance.
        """
        self.client = client

    def get_s3_bucket_name(self, url):
        parsed = urlparse(url)
        # Example: 'm-prod-ekascribe-batch.s3.amazonaws.com' → 'm-prod-ekascribe-batch'
        domain_parts = parsed.netloc.split('.')
        if 's3' in domain_parts:
            return domain_parts[0]
        return None


    def upload(self, file_paths, txn_id=None, action='default', extra_data={}, output_format={}):
        print("Uploading files to Ekascribe...")
        file_upload_result = self.client.files.upload(
            file_paths=file_paths,
            txn_id=txn_id,
            action=action,
            extra_data=extra_data,
            output_format=output_format
        )
        upload_info = self.client.files.get_last_upload_info()

        print("Uploading files to Ekascribe success...")
        if action == 'ekascribe-v2':
            try:
                bucket_name = self.get_s3_bucket_name(upload_info['uploadData']['url'])
                folder_path = upload_info['folderPath']
                
                s3_url = f"s3://{bucket_name}/{folder_path}"

                s3_file_paths = []

                for file in file_paths:
                    file_name = os.path.basename(file)
                    s3_file_paths.append(f"{s3_url}{file_name}")
                
                payload = {
                        "s3_url": s3_url,
                        "batch_s3_url": s3_url,
                        "additional_data": extra_data,
                        "mode": extra_data.get('mode'),
                        "input_language": output_format.get('input_language'),
                        "speciality": "speciality",
                        "Section": "section",
                        "output_format_template": output_format.get('output_template'),
                        "transfer": "non-vaded",
                        "client_generated_files": s3_file_paths
                    }
                auth_headers = {
                        "Authorization": f"Bearer {self.client.access_token}",
                    }
                resp = requests.post(
                    url=f"https://api.eka.care/voice/api/v2/transaction/init/{txn_id}",
                    headers=auth_headers,
                    json=payload
                )
                if resp.status_code != 201:
                    raise EkaScribeError(f"Upload initialisation failed: {resp.json()}")

            except Exception as e:
                raise EkaScribeError(f"Upload failed: {str(e)}")
        
        return file_upload_result

    def get_session_status(self, session_id: str, action="ekascribe") -> Dict[str, Any]:
        """
        Fetch the status and results of a voice recording session (Ekascribe job).

        After uploading an audio file for Ekascribe and receiving a webhook notification
        indicating completion, this method can be used to retrieve the transcription results.
        The `session_id` is typically part of the webhook payload.

        Args:
            session_id (str): The ID of the voice recording session.

        Returns:
            dict: A dictionary containing the session status information.
                  This includes the status of the job (e.g., "completed", "failed")
                  and, if successful, the output data, which often contains a
                  base64 encoded FHIR bundle.

        Raises:
            ValueError: If the session_id is null or empty.
            EkaCareAPIError: If the API call fails or returns an error status.

        Example:
            >>> # client is an instance of EkaCareClient
            >>> # session_id is obtained from the webhook after audio processing
            >>> status_response = client.v2rx.get_session_status("your_session_id_here")
            >>> print(f"Job Status: {status_response.get('status')}")
            >>> if status_response.get('status') == 'completed':
            ...     fhir_base64 = status_response.get('data', {}).get('output', {}).get('fhir')
            ...     if fhir_base64:
            ...         # Decode and process the FHIR data
            ...         pass
        """
        if not session_id:
            raise ValueError("Session ID cannot be null or empty")
        
        endpoint = f"/voice-record/api/status/{session_id}"
        
        if action == "ekascribe-v2":
            endpoint = f"voice/api/v3/status/{session_id}"

        return self.client.request(method="GET", endpoint=endpoint)


# ekacare/__init__.py
from .client import EkaCareClient
from .utils.exceptions import (
    EkaCareError, 
    EkaCareAPIError, 
    EkaCareAuthError, 
    EkaCareValidationError,
    EkaCareResourceNotFoundError
)

__all__ = [
    "EkaCareClient",
    "EkaCareError",
    "EkaCareAPIError",
    "EkaCareAuthError",
    "EkaCareValidationError",
    "EkaCareResourceNotFoundError"
]

# ekacare/auth/__init__.py
from .auth.auth import Auth
__all__ = ["Auth"]

from .tools.files import EkaFileUploader
__all__ = ["EkaFileUploader"]


# ekacare/records/__init__.py
from .records.records import Records
__all__ = ["Records"]

# ekacare/vitals/__init__.py
from .vitals.vitals import Vitals
__all__ = ["Vitals"]

# ekacare/utils/__init__.py
from .utils.exceptions import (
    EkaCareError, 
    EkaCareAPIError, 
    EkaCareAuthError, 
    EkaCareValidationError,
    EkaCareResourceNotFoundError
)

__all__ = [
    "EkaCareError",
    "EkaCareAPIError",
    "EkaCareAuthError",
    "EkaCareValidationError", 
    "EkaCareResourceNotFoundError"
]

# ekacare/abdm/__init__.py
from .abdm.profile import Profile

__all__ = ["Profile", "Consents", "CareContexts", "Enrollment"]


#class Voice:
#    def __init__(self, client):
#        self.client = client

# ekacare/notifications/__init__.py
class Notifications:
    def __init__(self, client):
        self.client = client



import requests
import os
import mimetypes
import boto3
from urllib.parse import urlencode, urlparse
import uuid
from io import BytesIO
import json


class EkaUploadError(Exception):
    """Upload related errors for Eka File Uploader"""
    pass

class EkaFileUploader:
    """
    Eka File Uploader SDK
    A simple and efficient way to handle authenticated file uploads to S3
    """
    def __init__(self, client):
        self.client = client
        self.__upload_info = None

    def get_s3_bucket_name(self, url):
        parsed = urlparse(url)
        # Example: 'm-prod-ekascribe-batch.s3.amazonaws.com' → 'm-prod-ekascribe-batch'
        domain_parts = parsed.netloc.split('.')
        if 's3' in domain_parts:
            return domain_parts[0]
        return None
    
    def get_upload_location(self, txn_id=None, action='ekascribe', extra_data={}):
        """
        Get S3 upload location
        
        Args:
            txn_id (str, optional): Transaction ID for grouping uploads
            
        Returns:
            dict: Upload location information
        """
        try:
            params = {}
            if txn_id:
                params['txn_id'] = txn_id
                params['action'] =  action
        
            response = self.client.request(
                method="POST",
                endpoint=f"/v1/file-upload?{urlencode(params)}",
                json=extra_data
            )
            return response
        except Exception as e:
            raise EkaUploadError(f"Error getting upload location: {str(e)}")
        
    def push_ekascribe_json(self, audio_files, txn_id, extra_data={}, upload_info={}, output_format = {}):
        s3_post_data = upload_info['uploadData']
        folder_path = upload_info['folderPath']
        s3_post_data['fields']['key'] = folder_path + txn_id + '.json'
        data = {
            "client-id": self.client.client_id,
            "transaction-id": txn_id,
            "audio-file": [],
            "transfer": "non-vaded"
        }
        for item in audio_files:
            data['audio-file'].append(item.split('/')[-1])
        data["additional_data"] = extra_data
        data["output_format"] = output_format
        data.update(extra_data)
        json_bytes = BytesIO(json.dumps(data, indent=2).encode('utf-8'))
        files = {'file': ('data.json', json_bytes.getvalue(), 'application/json')}

        response = requests.post(
            s3_post_data['url'],
            data=s3_post_data['fields'],
            files=files
        )
        if response.status_code != 204:
            raise EkaUploadError(f"Upload failed: {response.text}")
        return {
            'key': folder_path + txn_id + '.json',
            'contentType': 'application/json',
            'size': len(json_bytes.getvalue())
        }

    def upload(self, file_paths, txn_id=None, action='default',extra_data={}, output_format={}):
        """
        Upload a file to S3
        
        Args:
            file_path (str): Path to the file to upload
            txn_id (str, optional): Transaction ID for grouping uploads
            action (str, optional): Action to perform on the file, one of ['ekascribe', 'default']
            extra_data (dict, optional): Extra data to send with the upload request
            
        Returns:
            dict: Upload result containing key, content type, and size
        """
        try:
            return_list = []
            if not txn_id:
                txn_id = str(uuid.uuid4())
            upload_info = self.get_upload_location(txn_id, action=action, extra_data=extra_data)
            self.__upload_info = upload_info

            for file_path in file_paths: 
                file_size = os.path.getsize(file_path)
                if file_size > 100 * 1024 * 1024:  # 100MB threshold
                    return_list.append(self._upload_large_file(
                        upload_info['uploadData'],
                        upload_info['folderPath'],
                        file_path
                    ))
                else:
                    return_list.append(self._upload_single_file(
                        upload_info['uploadData'],
                        upload_info['folderPath'],
                        file_path
                    ))
            
            
            if action == 'ekascribe' or action =='ekascribe-v2':
                self.push_ekascribe_json(file_paths, txn_id, extra_data = extra_data, upload_info= upload_info, output_format=output_format)


            return return_list
        except Exception as e:
            raise EkaUploadError(f"Upload failed: {str(e)}")
        
    def _upload_single_file(self, upload_data, folder_path, file_path):
        """Internal method to handle single file upload"""
        file_name = os.path.basename(file_path)
        content_type = mimetypes.guess_type(file_path)[0] or 'application/octet-stream'
        
        s3_post_data = upload_data
        s3_post_data['fields']['key'] = folder_path + file_name
        
        try:
            with open(file_path, 'rb') as file:
                files = {
                    'file': (file_name, file, content_type)
                }
                
                # Optimized request configuration
                response = requests.post(
                    s3_post_data['url'],
                    data=s3_post_data['fields'],
                    files=files,
                    timeout=(30, 1800),  # 30s connect, 30min read timeout
                    stream=False,  # Let requests handle memory efficiently
                )
                
                if response.status_code != 204:
                    raise EkaUploadError(f"Upload failed for {file_name}: HTTP {response.status_code} - {response.text}")
                
                print(f"Successfully uploaded {file_name}")
                
                return {
                    'key': folder_path + file_name,
                    'contentType': content_type,
                    'size': os.path.getsize(file_path)
                }
                
        except requests.exceptions.Timeout:
            raise EkaUploadError(f"Upload timeout for {file_name}. File may be too large or connection too slow.")
        except requests.exceptions.ConnectionError:
            raise EkaUploadError(f"Connection error while uploading {file_name}. Check network connectivity.")
        except Exception as e:
            raise EkaUploadError(f"Unexpected error uploading {file_name}: {str(e)}")

    def _upload_large_file(self, upload_data, folder_path, file_path):
        """Internal method to handle large file upload using multipart"""
        file_name = os.path.basename(file_path)
        content_type = mimetypes.guess_type(file_path)[0] or 'application/octet-stream'
        file_size = os.path.getsize(file_path)
        
        s3_post_data = upload_data
        s3_post_data['fields']['key'] = folder_path + file_name
        
        try:
            # Use a custom file wrapper for progress tracking
            with ChunkedFileReader(file_path, chunk_size=8192) as chunked_file:
                files = {
                    'file': (file_name, chunked_file, content_type)
                }
                
                response = requests.post(
                    s3_post_data['url'],
                    data=s3_post_data['fields'],
                    files=files,
                    timeout=(30, 1800)
                )
                
                if response.status_code != 204:
                    raise EkaUploadError(f"Upload failed for {file_name}: HTTP {response.status_code} - {response.text}")
                
                print(f"Successfully uploaded {file_name}")
                
                return {
                    'key': folder_path + file_name,
                    'contentType': content_type,
                    'size': file_size
                }
                
        except Exception as e:
            raise EkaUploadError(f"Memory-efficient upload failed for {file_name}: {str(e)}")


    def get_last_upload_info(self):
        """
        Get the upload info from the most recent upload operation.
        
        Returns:
            dict: Upload info from the last successful upload, or None if no upload has been performed
        """
        if self.__upload_info is None:
            return None
        return self.__upload_info.copy()
    

class ChunkedFileReader:
    """
    File-like object that reads in chunks for memory efficiency
    """
    def __init__(self, file_path, chunk_size=8192):
        self.file_path = file_path
        self.chunk_size = chunk_size
        self.file = None
        self.total_size = os.path.getsize(file_path)
        self.bytes_read = 0
        
    def __enter__(self):
        self.file = open(self.file_path, 'rb')
        return self
        
    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.file:
            self.file.close()
    
    def read(self, size=-1):
        if not self.file:
            return b''
        
        if size == -1:
            size = self.chunk_size
        
        chunk = self.file.read(min(size, self.chunk_size))
        self.bytes_read += len(chunk)
        
        # Progress reporting (optional)
        if self.bytes_read % (1024 * 1024) == 0:  # Every MB
            progress = (self.bytes_read / self.total_size) * 100
            print(f"Progress: {progress:.1f}% ({self.bytes_read / (1024*1024):.1f}MB/{self.total_size / (1024*1024):.1f}MB)")
        
        return chunk
    
    def seek(self, offset, whence=0):
        if self.file:
            return self.file.seek(offset, whence)
    
    def tell(self):
        if self.file:
            return self.file.tell()
        return 0




import unittest
from unittest import mock
import json
import os
import sys

# Add the parent directory to the path so we can import the package
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from ekacare import EkaCareClient
from ekacare.utils.exceptions import EkaCareAPIError, EkaCareAuthError


class TestEkaCareClient(unittest.TestCase):
    """Tests for the EkaCare SDK client."""

    def setUp(self):
        """Set up test fixtures."""
        self.client = EkaCareClient(
            client_id="test_client_id",
            client_secret="test_client_secret",
            base_url="https://api.test.eka.care"
        )
        # Create a mock for the session to avoid actual API calls
        self.client.session = mock.MagicMock()
        
        # Mock successful response
        self.mock_response = mock.MagicMock()
        self.mock_response.ok = True
        self.mock_response.status_code = 200
        self.mock_response.headers = {'Content-Type': 'application/json'}
        self.mock_response.json.return_value = {"success": True}
        self.mock_response.text = json.dumps({"success": True})
        
        # Set the mock response as the return value for request methods
        self.client.session.request.return_value = self.mock_response

    def test_initialization(self):
        """Test client initialization."""
        self.assertEqual(self.client.client_id, "test_client_id")
        self.assertEqual(self.client.client_secret, "test_client_secret")
        self.assertEqual(self.client.base_url, "https://api.test.eka.care")
        
    def test_request_method(self):
        """Test the request method."""
        # Test a simple GET request
        result = self.client.request(
            method="GET",
            endpoint="/test/endpoint",
            auth_required=False
        )
        
        # Check the result
        self.assertEqual(result, {"success": True})
        
        # Verify the request was made correctly
        self.client.session.request.assert_called_once_with(
            method="GET",
            url="https://api.test.eka.care/test/endpoint",
            params=None,
            data=None,
            json=None,
            headers={"client-id": "test_client_id"},
            files=None
        )
        
    def test_auth_required_request(self):
        """Test request with auth token."""
        # Mock the access token
        self.client._access_token = "test_access_token"
        
        # Make a request that requires auth
        self.client.request(
            method="GET",
            endpoint="/auth/endpoint"
        )
        
        # Verify the Authorization header was included
        self.client.session.request.assert_called_once()
        call_kwargs = self.client.session.request.call_args[1]
        self.assertEqual(
            call_kwargs["headers"],
            {
                "Authorization": "Bearer test_access_token",
                "client-id": "test_client_id"
            }
        )
        
    def test_auth_error_handling(self):
        """Test handling of authentication errors."""
        # Mock a 401 response
        error_response = mock.MagicMock()
        error_response.ok = False
        error_response.status_code = 401
        error_response.text = "Unauthorized"
        
        self.client.session.request.return_value = error_response
        
        # Attempt to make a request that will fail with a 401
        with self.assertRaises(EkaCareAuthError):
            self.client.request(
                method="GET",
                endpoint="/secured/endpoint"
            )
            
    def test_api_error_handling(self):
        """Test handling of general API errors."""
        # Mock a 400 response
        error_response = mock.MagicMock()
        error_response.ok = False
        error_response.status_code = 400
        error_response.text = "Bad Request"
        
        self.client.session.request.return_value = error_response
        
        # Attempt to make a request that will fail with a 400
        with self.assertRaises(EkaCareAPIError):
            self.client.request(
                method="POST",
                endpoint="/some/endpoint",
                json={"invalid": "data"}
            )
            
    @mock.patch('ekacare.auth.auth.Auth.login')
    def test_access_token_auto_fetch(self, mock_login):
        """Test automatic fetching of access token."""
        # Mock the auth.login method
        mock_login.return_value = {"access_token": "new_access_token"}
        
        # Clear any existing access token
        self.client._access_token = None
        
        # Access the token property, which should trigger login
        token = self.client.access_token
        
        # Verify login was called and token was set
        mock_login.assert_called_once()
        self.assertEqual(token, "new_access_token")
        self.assertEqual(self.client._access_token, "new_access_token")
        
    def test_records_upload_document(self):
        """Test the records.upload_document method."""
        # Mock the get_authorization and session.post methods
        self.client.records.get_authorization = mock.MagicMock(return_value={
            "batch_response": [
                {
                    "document_id": "test_document_id",
                    "forms": [
                        {
                            "url": "https://upload.url",
                            "fields": {"field1": "value1"}
                        }
                    ]
                }
            ]
        })
        
        upload_response = mock.MagicMock()
        upload_response.ok = True
        self.client.session.post.return_value = upload_response
        
        # Create a temporary test file
        test_file = "test_file.pdf"
        with open(test_file, "w") as f:
            f.write("test content")
        
        try:
            # Call the upload_document method
            result = self.client.records.upload_document(
                file_path=test_file,
                document_type="ps",
                tags=["test"]
            )
            
            # Verify the result
            self.assertEqual(result, {"document_id": "test_document_id"})
            
            # Verify get_authorization was called with correct parameters
            self.client.records.get_authorization.assert_called_once()
            args = self.client.records.get_authorization.call_args[0][0]
            self.assertEqual(args[0]["dt"], "ps")
            self.assertEqual(args[0]["tg"], ["test"])
            
            # Verify session.post was called for the file upload
            self.client.session.post.assert_called_once()
            args = self.client.session.post.call_args[1]
            self.assertEqual(args["url"], "https://upload.url")
            self.assertEqual(args["data"], {"field1": "value1"})
            self.assertIn("files", args)
        finally:
            # Clean up the test file
            if os.path.exists(test_file):
                os.remove(test_file)


if __name__ == '__main__':
    unittest.main()


import os
from typing import Dict, Any, List, Optional

from ..utils.exceptions import EkaCareValidationError


class Records:
    """Client for interacting with Eka Care Records API."""
    
    def __init__(self, client):
        self.client = client
        
    def get_authorization(self, batch_request: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Obtain authorization for uploading documents.
        
        Args:
            batch_request (list): List of document requests
            
        Returns:
            dict: Response containing presigned URLs for upload
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> batch_request = [{
            ...     "dt": "lr",  # lab report
            ...     "dd_e": 1614556800,  # document date in epoch format
            ...     "tg": ["covid", "test"],  # tags
            ...     "files": [{
            ...         "contentType": "application/pdf",
            ...         "file_size": 1024000  # size in bytes
            ...     }]
            ... }]
            >>> auth_response = client.records.get_authorization(batch_request)
        """
        return self.client.request(
            method="POST",
            endpoint="/mr/api/v1/docs",
            json={"batch_request": batch_request}
        )
        
    def upload_document(
        self, 
        file_path: str, 
        document_type: str,
        document_date: Optional[int] = None,
        tags: Optional[List[str]] = None,
        title: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Upload a document to Eka Care.
        
        Args:
            file_path (str): Path to the file to upload
            document_type (str): Type of document (e.g., 'lr' for lab report, 'ps' for prescription)
            document_date (int, optional): Document date as epoch timestamp
            tags (list, optional): List of tags for the document
            title (str, optional): Title for the document
            
        Returns:
            dict: Response containing document ID
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> response = client.records.upload_document(
            ...     file_path="/path/to/lab_report.pdf",
            ...     document_type="lr",
            ...     tags=["covid", "test"],
            ...     title="COVID-19 Test Report"
            ... )
            >>> document_id = response["document_id"]
        """
        if not os.path.isfile(file_path):
            raise FileNotFoundError(f"File not found: {file_path}")
            
        # Get file size and content type
        file_size = os.path.getsize(file_path)
        content_type = self._get_content_type(file_path)
        
        # Prepare batch request
        batch_request = [{
            "dt": document_type,
            "dd_e": document_date,
            "tg": tags or [],
            "files": [
                {
                    "contentType": content_type,
                    "file_size": file_size
                }
            ]
        }]
        
        if title:
            batch_request[0]["title"] = title
        
        # Get presigned URL for upload
        auth_response = self.get_authorization(batch_request)
        
        # Upload the file using the presigned URL
        batch_response = auth_response.get("batch_response", [])
        if not batch_response:
            raise EkaCareValidationError("No upload URL received")
            
        document_id = batch_response[0].get("document_id")
        forms = batch_response[0].get("forms", [])
        
        if not forms:
            raise EkaCareValidationError("No upload forms received")
            
        form = forms[0]
        upload_url = form.get("url")
        fields = form.get("fields", {})
        
        # Prepare the form data for the upload
        with open(file_path, "rb") as file:
            files = {"file": (os.path.basename(file_path), file, content_type)}
            
            # Use requests session directly to upload to the presigned URL
            response = self.client.session.post(
                url=upload_url,
                data=fields,
                files=files
            )
            
        if not response.ok:
            raise EkaCareValidationError(f"Upload failed: {response.text}")
            
        return {"document_id": document_id}
        
    def list_documents(
        self,
        updated_after: Optional[int] = None,
        next_token: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        List documents.
        
        Args:
            updated_after (int, optional): Filter documents updated after this timestamp
            next_token (str, optional): Token for pagination
            
        Returns:
            dict: Response containing list of documents
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> documents = client.records.list_documents()
            >>> for doc in documents["items"]:
            ...     print(doc["record"]["item"]["document_id"])
        """
        params = {}
        if updated_after:
            params["u_at__gt"] = updated_after
        if next_token:
            params["offset"] = next_token
            
        return self.client.request(
            method="GET",
            endpoint="/mr/api/v1/docs",
            params=params,
            headers={"accept": "application/json"}
        )
        
    def get_document(self, document_id: str) -> Dict[str, Any]:
        """
        Get details of a specific document.
        
        Args:
            document_id (str): ID of the document
            
        Returns:
            dict: Document details
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> document = client.records.get_document("document123")
            >>> print(document["document_type"])
        """
        return self.client.request(
            method="GET",
            endpoint=f"/mr/api/v1/docs/{document_id}"
        )
        
    def update_document(
        self,
        document_id: str,
        document_type: Optional[str] = None,
        document_date: Optional[int] = None,
        tags: Optional[List[str]] = None,
        ndhm: Optional[bool] = None,
        oid: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Update a document's metadata.
        
        Args:
            document_id (str): ID of the document
            document_type (str, optional): Type of document (e.g., 'lr' for lab report)
            document_date (int, optional): Document date as epoch timestamp
            tags (list, optional): List of tags for the document
            ndhm (bool, optional): Whether to link the document to NDHM
            oid (str, optional): OID to associate with the document
            
        Returns:
            dict: Empty response on success
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> client.records.update_document(
            ...     document_id="document123",
            ...     document_type="ps",
            ...     tags=["medication"]
            ... )
        """
        data = {}
        if document_type:
            data["dt"] = document_type
        if document_date:
            data["dd_e"] = document_date
        if tags:
            data["tg"] = tags
        if ndhm is not None:
            data["ndhm"] = ndhm
        if oid:
            data["oid"] = oid
            
        return self.client.request(
            method="PATCH",
            endpoint=f"/mr/api/v1/docs/{document_id}",
            json=data
        )
        
    def delete_document(self, document_id: str) -> Dict[str, Any]:
        """
        Delete a document.
        
        Args:
            document_id (str): ID of the document
            
        Returns:
            dict: Empty response on success
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> client.records.delete_document("document123")
        """
        return self.client.request(
            method="DELETE",
            endpoint=f"/mr/api/v1/docs/{document_id}"
        )
    
    def retrieve_health_records(self, 
                               identifier: str, 
                               hip_id: Optional[str] = None, 
                               health_id: Optional[str] = None) -> Dict[str, Any]:
        """
        Retrieve health records in FHIR format.
        
        Args:
            identifier (str): Care context ID
            hip_id (str, optional): Health Information Provider ID
            health_id (str, optional): ABHA address
            
        Returns:
            dict: FHIR bundle containing health records
            
        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret")
            >>> records = client.records.retrieve_health_records(
            ...     identifier="care_context_123",
            ...     hip_id="hip123",
            ...     health_id="user@abdm"
            ... )
        """
        params = {"identifier": identifier}
        if hip_id:
            params["hip_id"] = hip_id
        if health_id:
            params["health_id"] = health_id
            
        return self.client.request(
            method="GET",
            endpoint="/health/api/v1/fhir/retrieve",
            params=params
        )
        
    def _get_content_type(self, file_path: str) -> str:
        """Determine content type based on file extension."""
        ext = os.path.splitext(file_path)[1].lower()
        content_types = {
            ".jpg": "image/jpeg",
            ".jpeg": "image/jpeg",
            ".png": "image/png",
            ".pdf": "application/pdf"
        }
        return content_types.get(ext, "application/octet-stream")



from typing import Dict, Any


class ClinicAndDoctor:
    """Client for getting clinic details"""

    def __init__(self, client):
        self.client = client

    def get_clinic_details(self, clinic_id: str) -> Dict[str, Any]:
        """
        Fetch the details of a specific appointment.

        Args:
            clinic_id (str): The ID of the appointment to fetch

        Returns:
            dict: Clinic details

        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> clinic = client.clinic_doctor.get_clinic_details("YOUR_CLINIC_ID")
            >>> print(clinic)
        """
        return self.client.request(
            method="GET",
            endpoint=f"/dr/v1/business/clinic/{clinic_id}"
        )


    def get_doctor_details(self, doctor_id: str) -> Dict[str, Any]:
        """
        Fetch the details of a specific appointment.

        Args:
            doctor_id (str): The ID of the appointment to fetch

        Returns:
            dict: Clinic details

        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> clinic = client.clinic_doctor.get_doctor_details("YOUR_DOCTOR_ID")
            >>> print(clinic)
        """
        return self.client.request(
            method="GET",
            endpoint=f"/dr/v1/doctor/{doctor_id}"
        )



from pathlib import Path

from setuptools import find_packages, setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="ekacare",
    version="0.0.9",
    packages=find_packages(),
    install_requires=[
        "requests==2.25.0",
        "pyjwt==2.0.0",
        "boto3==1.37.0"
    ],
    author="Eka Care SDK Developer",
    author_email="developer@eka.care",
    description="Python SDK for Eka Care APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="healthcare, eka care, api, sdk, health records, abdm",
    url="https://developer.eka.care",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.6",
)



import requests
from urllib.parse import urljoin
from typing import Dict, Any, Optional, Union

from .appointments.appointments import Appointments
from .doctor_clinic.clinic_doctor import ClinicAndDoctor
from .patient.patient import Patient
from .utils.exceptions import EkaCareAPIError, EkaCareAuthError
from .auth.auth import Auth
from .records.records import Records
from .tools.files import EkaFileUploader
from .vitals.vitals import Vitals
from .abdm.profile import Profile
from .webhooks.appointment import AppointmentWebhook
from .v2rx.v2rx import V2RX


class EkaCareClient:
    """
    Main client for interacting with the Eka Care API.
    
    Args:
        client_id (str): Your Eka Care API client ID
        client_secret (str): Your Eka Care API client secret
        base_url (str, optional): Base URL for the Eka Care API. Defaults to the production API URL.
    """
    
    def __init__(
        self, 
        client_id: str,
        client_secret: str,
        base_url: str = "https://api.eka.care",
        api_key: Optional[str] = None,
        access_token: Optional[str] = None
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = base_url
        self.session = requests.Session()
        self._access_token = access_token
        self.api_key = api_key
        
        # Initialize API modules
        self.auth = Auth(self)
        self.records = Records(self)
        self.files = EkaFileUploader(self)
        self.vitals = Vitals(self)
        self.abdm_profile = Profile(self)
        self.appointments = Appointments(self)
        self.patient  = Patient(self)
        self.clinic_doctor = ClinicAndDoctor(self)
        self.appointment_webhook = AppointmentWebhook(self)
        self.v2rx = V2RX(self)
        
    @property
    def access_token(self) -> str:
        """Get the current access token or request a new one if needed."""
        if not self._access_token:
            token_response = self.auth.login()
            self._access_token = token_response["access_token"]
        return self._access_token
    
    def set_access_token(self, token: str) -> None:
        """Manually set the access token."""
        self._access_token = token
    
    def request(
        self, 
        method: str, 
        endpoint: str, 
        params: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        files: Optional[Dict[str, Any]] = None,
        auth_required: bool = True,
        raw_response: bool = False
    ) -> Union[Dict[str, Any], bytes, requests.Response]:
        """
        Make a request to the Eka Care API.
        
        Args:
            method (str): HTTP method (GET, POST, PUT, DELETE, etc.)
            endpoint (str): API endpoint path
            params (dict, optional): Query parameters
            data (dict, optional): Form data
            json (dict, optional): JSON body
            headers (dict, optional): Additional headers
            files (dict, optional): Files to upload
            auth_required (bool, optional): Whether auth token is required. Defaults to True.
            raw_response (bool, optional): Return the raw response object. Defaults to False.
            
        Returns:
            Union[dict, bytes, Response]: Response data, binary content, or response object
            
        Raises:
            EkaCareAPIError: If the API returns an error
            EkaCareAuthError: If authentication fails
        """
        url = urljoin(self.base_url, endpoint)
        
        if headers is None:
            headers = {}
            
        if auth_required:
            headers["Authorization"] = f"Bearer {self.access_token}"
            
        if self.client_id:
            headers["client-id"] = self.client_id
        
        response = self.session.request(
            method=method,
            url=url,
            params=params,
            data=data,
            json=json,
            headers=headers,
            files=files
        )
        
        if raw_response:
            return response
            
        if not response.ok:
            if response.status_code == 401:
                raise EkaCareAuthError(
                    f"Authentication error: {response.status_code} - {response.text}"
                )
            raise EkaCareAPIError(
                f"API error {response.status_code}: {response.text}"
            )
            
        # Handle different response content types
        content_type = response.headers.get('Content-Type', '')
        
        if 'application/json' in content_type:
            return response.json()
        elif 'image/' in content_type or 'application/pdf' in content_type:
            return response.content
        elif not response.content:
            return {'status': 'success', 'status_code': response.status_code}
        else:
            try:
                return response.json()
            except ValueError:
                return response.text



from typing import Dict, Any


class Appointments:
    """Client for interacting with Eka Appointment System"""

    def __init__(self, client):
        self.client = client

    def get_appointment_details(self, appointment_id: str) -> Dict[str, Any]:
        """
        Fetch the details of a specific appointment.

        Args:
            appointment_id (str): The ID of the appointment to fetch

        Returns:
            dict: Appointment details

        Example:
            >>> client = EkaCareClient(client_id="your_id", client_secret="your_secret", access_token="user_token")
            >>> appointment = client.appointments.get_appointment_details("YOUR_APPOINTMENT_ID")
            >>> print(appointment)
        """
        return self.client.request(
            method="GET",
            endpoint=f"/dr/v1/appointment/{appointment_id}"
        )
